using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using Microsoft.Samples.ServiceHosting.StorageClient;
using System.Security.Cryptography;
using System.Text;
using System.IO;
using System.Collections.Specialized;

namespace wazdrop_WebRole.Controllers
{
    public class WazDropController : Controller
    {
        public static string MakeBlobReadSignature(DateTime starttime, DateTime endtime, string account, string container, string blobname)
        {
            string stringtosign = string.Format("r\n{0:yyyy-MM-ddTHH:mm:ssZ}\n{1:yyyy-MM-ddTHH:mm:ssZ}\n/{2}/{3}/{4}\n", starttime, endtime, account, container, blobname);
            using (var hmac = new HMACSHA256(Convert.FromBase64String(StorageAccountInfo.GetDefaultBlobStorageAccountFromConfiguration().Base64Key)))
            {
                return Convert.ToBase64String(hmac.ComputeHash(Encoding.UTF8.GetBytes(stringtosign)));
            }
        }

        public ActionResult Index()
        {
            var accountInfo = StorageAccountInfo.GetDefaultBlobStorageAccountFromConfiguration();
            var container = BlobStorage.Create(accountInfo).GetBlobContainer("files");
            container.DeleteContainer();
            return View();
        }

        public ActionResult Upload()
        {
            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Uploaded(int minutes)
        {
            var file = Request.Files[0];
            var accountInfo = StorageAccountInfo.GetDefaultBlobStorageAccountFromConfiguration();
            var container = BlobStorage.Create(accountInfo).GetBlobContainer("files");
            container.CreateContainer(); // NOTE: This should REALLY only be done once, before deploying the application.
            var metadata = new NameValueCollection();
            metadata["originalname"] = Path.GetFileName(file.FileName);
            var props = new BlobProperties(Guid.NewGuid().ToString()) { ContentType = file.ContentType, Metadata = metadata };
            container.CreateBlob(props, new BlobContents(file.InputStream), false);
            var uri = new UriBuilder();
            uri.Scheme = "https";
            uri.Host = string.Format("{0}.{1}", accountInfo.AccountName, container.BaseUri.Host);
            uri.Path = string.Format("files/{0}", props.Name);
            DateTime starttime = DateTime.UtcNow;
            DateTime endtime = DateTime.UtcNow + TimeSpan.FromMinutes(minutes);
            uri.Query = string.Format("st={0}&se={1}&sr=b&sp=r&sig={2}", Uri.EscapeDataString(starttime.ToString("yyyy-MM-ddTHH:mm:ssZ")), Uri.EscapeDataString(endtime.ToString("yyyy-MM-ddTHH:mm:ssZ")), Uri.EscapeDataString(MakeBlobReadSignature(starttime, endtime, accountInfo.AccountName, "files", props.Name)));
            ViewData["url"] = uri.ToString();
            ViewData["minutes"] = minutes;
            return View();
        }
    }
}